--[[
	Offers functionality for the auction house, needed by Vendor.
	This includes the addition of new tabs and specifying the 
	tab to be selected when opening the auction house frame.

	Copyright (C) Udorn (Blackhand)

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
--]]

vendor.AuctionHouse = vendor.Vendor:NewModule("AuctionHouse", "AceEvent-2.0", "AceHook-2.1");
local L = vendor.Locale.GetInstance()
local waterFall = AceLibrary("Waterfall-1.0");
local self = vendor.AuctionHouse;

local BLINK_INTERVAL = 1.5

local START_TABS = {
	[L["Browse"]] = 1,
	[L["Bids"]] = 2,
	[L["Auctions"]] = 3, 
	[L["AuctionMaster"]] = 4,
}

--[[
	Toggles the position of the AuctionMaster button.
--]]
local function _ToggleButtonPosition(but, arg1, arg2, initPos)
	local self = arg1
	self.but:ClearAllPoints()
	if (self.isLeftPos or initPos) then
		self.but:SetPoint("TOPRIGHT", AuctionFrame, "TOPRIGHT", -21, -13);
		self.isLeftPos = false
	else
		self.but:SetPoint("TOPLEFT", AuctionFrame, "TOPLEFT", 67, -13);
		self.isLeftPos = true
	end
end

--[[
	Sets the given value in the profile.
--]]
local function _SetValue(field, value)
	self.db.profile[field] = value;
	self:ApplySettings();
end

--[[
	Returns the given value from the profile.
--]]
local function _GetValue(field)
	return self.db.profile[field]
end

-- describe the commands contained in the main menu
local CMDS = {
	scan = {
		type = "execute",
		text = L["Scan"],
		func = function() vendor.Scanner:Scan() end,
		disabledFunc = function() return vendor.Scanner:IsScanning() end,
		order = 1
	},
	fastScan = {
		type = "execute",
    	text = L["Fast scan"],
    	func = function() vendor.Scanner:Scan(nil, true) end,
    	disabledFunc = function() return vendor.Scanner:IsScanning() end,
    	order = 2
	},
	snipe = {
		type = "execute",
		text = L["Snipe"],
		func = function() vendor.Sniper:OpenSnipeDialog() end,
		order = 3
	},
	conf = {
		type = "execute",
		text = L["Configuration"],
		func = function() 
			if (waterFall:IsOpen("AuctionMaster")) then 
				waterFall:Close("AuctionMaster"); 
			else
				waterFall:Open("AuctionMaster");
			end
		end,
		order = 4
	},
	help = {
		type = "execute",
		text = L["Help"],
		func = function() vendor.Help:Show(vendor.Help.DOCUMENTATION) end,
		order = 5
	},
	togglePos = {
		type = "execute",
		text = L["Toggle position"],
		func = _ToggleButtonPosition,
		order = 20
	},
}

-- describe the fast commands on the portrait icon
local FAST_CMDS = {
	normalScan = {
		type = "multiselect",
		values = {
			["normalScanSelected"] = L["Scan"];
			["fastScanSelected"] = L["Fast scan"]; 
		},
		get = _GetValue,
		set = _SetValue,
		order = 1
	},
}

--[[
	Options for this module.
--]]
local function _CreateOptions()
	vendor.Vendor.options.args.AuctionHouse = {
		type = "group",
		name = L["AuctionHouse"],
		desc = L["AuctionHouse"],
		args = {
			toggle = {
				type = "text",
				name = L["Start tab"],
				desc = L["Tab to be selected when opening the auction house."],
				get = _GetValue,
				set = _SetValue,
				validate = {L["Browse"], L["Bids"], L["Auctions"], L["AuctionMaster"]},
				passValue = "startTab",
			},
		}
	}
end

--[[
	Called for the tab button.
--]]
local function _TabOnClick(self, id)
	vendor.Vendor:Debug("TabOnClick enter id: "..id);
	-- hide all custom tabs
	--self.buttonFrame:Show()
	self.but:Show()
	local unknownTab = false
	for _, frame in pairs(self.frames) do
		frame:HideTabFrame();
	end
	if (self.frames[id]) then
		-- blizzard's' frames, set tab and play sound
		PanelTemplates_SetTab(AuctionFrame, id);
		AuctionFrame.type = self.frames[id]:GetTabType();
		
		AuctionFrameAuctions:Hide();
		AuctionFrameBrowse:Hide();
		AuctionFrameBid:Hide();
		PlaySound("igCharacterInfoTab");
		self.frames[id]:UpdateTabFrame();
		self.frames[id]:ShowTabFrame();
	elseif (id > 3) then
		unknownTab = true
		--self.buttonFrame:Hide()
		self.but:Hide()
	end	
	if (id == 3) then
		-- there is a bug in blizzard's code: they don't set the tab for "owner"
		self.type = "owner"
	elseif (unknownTab) then
		self.type = "???"
	else
		self.type = AuctionFrame.type
	end
	vendor.Vendor:Debug("set to type: "..self.type)
end

--[[
	Create the main menu dropdown.
--]]
local function _CreateButton(self)
	local but = CreateFrame("Button", nil, AuctionFrame, "UIPanelButtonTemplate")
	but:SetText(L["AuctionMaster"])
	but:SetWidth(120)
	but:SetHeight(22)
	self.but = but;
	_ToggleButtonPosition(nil, self, nil, true)
	but.ctrl = self
	but:SetScript("OnClick", function(but) but.ctrl.mainMenu:Show() end)
end

--[[
	Returns the frame for the given tab number
--]]
local function _GetTabFrame(self, id)
	if (id <= 3) then
		return getglobal("AuctionFrameTab"..id);
	end
	return self.frames[id];
end

--[[
	The portrait button has been clicked.
--]]
local function _OnClick(self, button)
	vendor.Vendor:Debug("_OnClick button: %s", button)
	if (button == "LeftButton") then
    	if (vendor.Scanner:IsScanning()) then
    		vendor.Scanner:StopScan()
    	elseif (self.db.profile.fastScanSelected) then
    		vendor.Scanner:Scan(nil, true)
    	elseif (self.db.profile.normalScanSelected) then
    		vendor.Scanner:Scan()
    	else
    		message(L["Please select a scan mode first. You can do that with a right click on the portrait icon."])	
    	end
    elseif (button == "RightButton") then
    	self.portraitMenu:Show()
    end
end

--[[
	Update the portrait for blinking effect.
--]]
local function _OnUpdate(self, diff)
	local doBlink = 
		vendor.Scanner:IsScanning() or
		(not self.db.profile.fastScanSelected and not self.db.profile.normalScanSelected) 
	if (doBlink) then
		-- OnLeave will reset it, so we do it each time 
		self.logoTexture:SetTexture("Interface\\Addons\\AuctionMaster\\Media\\portrait_orange_highlight")
		if (not self.blinkTime) then
			self.blinkTime = 0.0
		end
		self.blinkTime = self.blinkTime + diff
		-- the blinking is a like a triangle with twice the blinking interval as "length"
		local tri = math.fmod(self.blinkTime, BLINK_INTERVAL * 2.0)
		-- the shorter the diff from BLINK_INTERVAL, the more transparent the portrait will be
		local diff =  math.abs(tri - BLINK_INTERVAL)
		-- finally get a value between 0 and 1
		self.logoTexture:SetAlpha(diff / BLINK_INTERVAL)
	else
		if (self.blinkTime) then
			vendor.Vendor:Debug("_OnUpdate set normal texture")
			self.logoTexture:SetTexture("Interface\\Addons\\AuctionMaster\\Media\\portrait_orange")
		end
		self.blinkTime = nil
		self.logoTexture:SetAlpha(1)
	end
end

--[[
	Creates the AuctionMaster texture.
--]]
local function _CreateAuctionMasterLogo(self)
	vendor.Vendor:Debug("_CreateAuctionMasterLogo enter")
	self.logo = CreateFrame("Button", "VendorLogo", AuctionFrame)
	self.logo:SetWidth(55)
	self.logo:SetHeight(55)
	self.logo:SetPoint("TOPLEFT", 10, -8)
	self.logo:SetToplevel(true)
	self.logo.ctrl = self
	self.logo:RegisterForClicks("LeftButtonUp", "RightButtonUp")
	self.logo:SetScript("OnClick", function(frame, button) _OnClick(frame.ctrl, button) end)
	self.logo.tooltipText = L["Left click for starting/stopping scan.\nRight click for selecting preferred scan mode."]
	
	self.logoBackground = self.logo:CreateTexture("VendorLogoBackground", "BACKGROUND")
	self.logoBackground:SetWidth(55)
	self.logoBackground:SetHeight(55)
	self.logoBackground:SetTexCoord(0, 1, 0, 1)
	self.logoBackground:SetPoint("TOPLEFT", 0, 0)
	self.logoBackground:SetTexture("Interface\\Addons\\AuctionMaster\\Media\\portrait_orange_dark")
	
	self.logoTexture = self.logo:CreateTexture("VendorLogoTexture", "ARTWORK")
	self.logoTexture:SetWidth(55)
	self.logoTexture:SetHeight(55)
	self.logoTexture:SetTexCoord(0, 1, 0, 1)
	self.logoTexture:SetPoint("TOPLEFT", 0, 0)
	self.logoTexture:SetTexture("Interface\\Addons\\AuctionMaster\\Media\\portrait_orange")
	
	self.logoLetters = self.logo:CreateTexture("VendorLogoLetters", "OVERLAY")
	self.logoLetters:SetWidth(55)
	self.logoLetters:SetHeight(55)
	self.logoLetters:SetTexCoord(0, 1, 0, 1)
	self.logoLetters:SetPoint("TOPLEFT", 1, 2)
	self.logoLetters:SetTexture("Interface\\Addons\\AuctionMaster\\Media\\letters_bright")

	-- handle highlighting of portrait
	self.logo:SetScript("OnEnter", function(frame)
		GameTooltip_SetDefaultAnchor(GameTooltip, frame) 
		GameTooltip:SetText(frame.tooltipText, 1, 1, 1, 1, true) 
		self.blinkTime = nil; frame.ctrl.logoTexture:SetTexture("Interface\\Addons\\AuctionMaster\\Media\\portrait_orange_highlight") 
	end)
	self.logo:SetScript("OnLeave", function(frame) 
		GameTooltip:Hide()
		frame.ctrl.logoTexture:SetTexture("Interface\\Addons\\AuctionMaster\\Media\\portrait_orange") 
	end)
	
	-- set blinking effect
	self.logo:SetScript("OnUpdate", function(frame, arg1) _OnUpdate(frame.ctrl, arg1) end)
	
	vendor.Vendor:Debug("_CreateAuctionMasterLogo exit")
end

--[[
	Initializes the module.
--]]
function vendor.AuctionHouse:OnInitialize()
	self.db = vendor.Vendor:AcquireDBNamespace("AuctionHouse");
	vendor.Vendor:RegisterDefaults("AuctionHouse", "profile", {
		startTab = L["Browse"],
		fastScanSelected = false,
		normalScanSelected = false,
	})
	_CreateOptions();
end

--[[
	Initializes the module.
--]]
function vendor.AuctionHouse:OnEnable()
	vendor.Vendor:Debug("AuctionHouse:OnEnable enter")
	self.frames = {};
	self:RegisterEvent("AUCTION_HOUSE_SHOW");
	self:SecureHook("AuctionFrameTab_OnClick");
	_CreateButton(self)
	self.mainMenu = vendor.PopupMenu:new("VendorMainMenu", CMDS, self)
	self.portraitMenu = vendor.PopupMenu:new("VendorPortraitMenu", FAST_CMDS, self)
	_CreateAuctionMasterLogo(self)
	vendor.Vendor:Debug("AuctionHouse:OnEnable exit")
end

--[[
	Hooks opening of the auction house to select start tab.
--]]
function vendor.AuctionHouse:AUCTION_HOUSE_SHOW()
	if (self.startTab) then
		--PanelTemplates_SetTab(AuctionFrame, self.startTab);
		-- the first parameter is unused
		AuctionFrameTab_OnClick(_GetTabFrame(self, self.startTab), self.startTab);
	end
	if (not self.db.profile.lastSeenRelease or self.db.profile.lastSeenRelease ~= vendor.Vendor.version) then
		vendor.Help:Show(vendor.Help.RELEASES_3X)
		UIErrorsFrame:AddMessage(L["New AuctionMaster release available"], nil, nil, nil, nil, UIERRORS_HOLD_TIME)
		self.db.profile.lastSeenRelease = vendor.Vendor.version
	end
end

--[[
	Hook for auction frame tab clicks
--]]
function vendor.AuctionHouse:AuctionFrameTab_OnClick(frame, id)
	_TabOnClick(self, id or frame:GetID());
end

--[[
	Returns true, if the currently opened auction house is neutral.
	If no auction house is open, false will be returned.
--]]
function vendor.AuctionHouse:IsNeutral()
	if (not AuctionFrame or not AuctionFrame:IsVisible()) then
		return false;
	end
	return GetAuctionHouseDepositRate() ~= 5;
end

--[[
	Constructor for tabbed frames.
	@param name the name of the created frame
	@id the tab sequence starting with 4
	@tabFrame the controler for the created frame. It has to implement the following methods:
	
	UpdateTabFrame: Updates the frame for displaying it. This is the right place to 
	switch the background.

	ShowTabFrame: Displays the frame object.
		
	HideTabFrame: Hides the frame object.
	
	GetTabType: Returns an unqiue string identifier like "vendor" to be used
	for GetCurrentAuctionHouseTab.
--]]
function vendor.AuctionHouse:CreateTabFrame(name, title, tabTitle, tabFrame)
	vendor.Vendor:Debug("CreateTabFrame");
	local frame = CreateFrame("Frame", name, AuctionFrame);
	frame:SetWidth(758);
	frame:SetHeight(447);
	frame:SetPoint("TOPLEFT");
	local f = frame:CreateFontString("BrowseTitle", "BACKGROUND", "GameFontNormal");
	f:SetText(title);
	f:SetPoint("TOP", 0, -18);
	frame:Hide();
	local id = AuctionFrame.numTabs + 1
	self.frames[id] = tabFrame;
	-- create the tab button
	vendor.Vendor:Debug("CreateTabFrame numTabs: "..AuctionFrame.numTabs)
	local but = CreateFrame("Button", "AuctionFrameTab"..id, AuctionFrame, "AuctionTabTemplate");
	vendor.Vendor:Debug("CreateTabFrame created tab button with id: "..id)
	but:SetID(id);
	but:SetText(tabTitle);
	but:SetPoint("TOPLEFT", "AuctionFrameTab"..(id - 1), "TOPRIGHT", -8, 0);
	but.ctrl = self
	--but:SetScript("OnClick", function(but) _TabOnClick(but.ctrl, but:GetID()) end)
	but:Hide(); -- don't know, why it's needed!?
	but:Show();
	PanelTemplates_SetNumTabs(AuctionFrame, id)
	PanelTemplates_EnableTab(AuctionFrame, id)	
	return frame;
end

--[[
	Constructor for close button of the tabs.
--]]
function vendor.AuctionHouse:CreateCloseButton(parent, name)
	local but = CreateFrame("Button", name, parent, "UIPanelButtonTemplate");
	but:SetText(L["Close"]);
	but:SetWidth("80");
	but:SetHeight("22");
	but:SetPoint("BOTTOMRIGHT", parent, "BOTTOMRIGHT", 66, 14);
	but:SetScript("OnClick", function() CloseAuctionHouse(); end);
	return but;
end

--[[
	Returns the type of tab currently shown in the auction house. Possible 
	values are the ones from blizzard "list", "bidder", "owner" and
	the vendor defined ones. The Tabs of vendor have to implement the
	method GetTabType to retrieve it from them.
--]]
function vendor.AuctionHouse:GetCurrentAuctionHouseTab()
	return self.type;
end

--[[
	Returns whether the given tab is currently shown.
--]]
function vendor.AuctionHouse:IsAuctionHouseTabShown(tabType)
	vendor.Vendor:Debug("IsAuctionHouseTabShown tabType: "..tabType)
	if (AuctionFrame and AuctionFrame:IsVisible()) then
		vendor.Vendor:Debug("type: "..self.type)
		return self.type == tabType
	end
	vendor.Vendor:Debug("return false")
	return false
end

--[[
	Applys the possibly changed settings.
--]]
function vendor.AuctionHouse:ApplySettings()
	self.startTab = START_TABS[self.db.profile.startTab] or 1;
end

