--[[
	Adds a new tab to the auction house for more convinient selling of items.
	Enables the selling of many items at once, prefills the desired prizes and
	remembers them.
	
	Copyright (C) Udorn (Blackhand)
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.	
--]]

vendor.Seller = vendor.Vendor:NewModule("Seller", "AceEvent-2.0", "AceHook-2.1");
local L = vendor.Locale.GetInstance()
local self = vendor.Seller;

vendor.Seller.PRIZE_MODEL_FIX = 1
vendor.Seller.PRIZE_MODEL_MARKET = 2
vendor.Seller.PRIZE_MODEL_CURRENT = 3
vendor.Seller.PRIZE_MODEL_UNDERCUT = 4
vendor.Seller.PRIZE_MODEL_MULTIPLIER = 5
vendor.Seller.PRIZE_MODEL_LOWER = 6

local SELLER_TAB = "seller"
local INVENTORY_LINE_HEIGHT = 27
local INVENTORY_LINES = 11
local BID_TYPE_PER_ITEM = 1
local BID_TYPE_PER_STACK = 2
local BID_TYPE_ALL = 3
local DURATIONS = {720, 1440, 2880}
local DURATION_CONSTS = {["12h"] = 1; ["24h"] = 2; ["48h"] = 3}
local STACK_DROPDOWN_ID = 1
local COUNT_DROPDOWN_ID = 2
local PRIZING_DROPDOWN_ID = 3
local BIDTYPE_DROPDOWN_ID = 4
local DURATION_DROPDOWN_ID = 5
local MARKET_PERCENT = 2

--[[
	The refresh button has been clicked. A scan will be triggered.
--]]
local function _RefreshClicked(but) 
	if (self.item) then
		vendor.Scanner:Scan(self.item.link);
	elseif (self.lastItem) then
		vendor.Scanner:Scan(self.lastItem.link);
	end
end

--[[
	Tries to retrieve a link for an item with the given name in the inventory.
--]]
local function _FindInventoryItemLink(name)
   	for bag = 0, 4 do
      	for slot = 1, GetContainerNumSlots(bag) do
	 		local itemLink = GetContainerItemLink(bag, slot);
	 		if (itemLink) then
	    		local itemName = GetItemInfo(itemLink);
	    		if (name and name == itemName) then
	    			return itemLink; -- don't like this, but we need to get out of two loops
	    		end
	 		end
      	end
   	end
   	return nil;
end

--[[
	Returns "minBid, buyOut" for the given item, if known.
--]]
local function _GetItemAuctionSellValues(itemLink)
	local itemLinkKey = vendor.Items:GetItemLinkKey(itemLink);
	local itemInfo = vendor.Items:GetItemInfo(itemLinkKey, vendor.AuctionHouse:IsNeutral());
	if (itemInfo) then
		return itemInfo.minBid, itemInfo.buyout;
	end
	return nil;
end

--[[
	Returns "avgMinBid, avgBuyout" for the given item, if known.
--]] 
local function _GetItemAuctionAvgValues(itemLink)
	local itemLinkKey = vendor.Items:GetItemLinkKey(itemLink);
	local itemInfo = vendor.Items:GetItemInfo(itemLinkKey, vendor.AuctionHouse:IsNeutral());
	if (itemInfo) then
		return itemInfo.avgMinBid, itemInfo.avgBuyout;
	end
	return nil;
end

--[[
	Modifies the given minBid and buyout prices by the given modifier.
--]]
local function _ModifyPrices(minBid, buyout, minBidPercent, buyoutPercent)
	vendor.Vendor:Debug("_ModifyPrices minBid: "..(minBid or "NA").." buyout: "..(buyout or "NA").." mPer: "..minBidPercent.." bPer: "..buyoutPercent)
	if (minBid and minBid > 0) then
		minBid = math.floor(minBid * (minBidPercent / 100.0) + 0.5);
		minBid = math.max(1, minBid)
		if (buyout and buyout > 0) then
			buyout = math.floor(buyout * (buyoutPercent / 100.0) + 0.5);
			buyout = math.max(1, buyout)
		end
	end
	if (minBid and buyout and buyout > 0 and minBid > buyout) then
		buyout = minBid; 
	end
	return minBid, buyout;	
end

--[[
	Returns "undergo, percent" for given price model.
--]]
local function _GetPriceModelModifier(self, priceModel)
	local minBidPercent = 0
	local buyoutPercent = 0
	if (priceModel == self.PRIZE_MODEL_CURRENT) then
		minBidPercent = self.db.profile.currentMinBidMul;
		buyoutPercent = self.db.profile.currentBuyoutMul;
	elseif (priceModel == self.PRIZE_MODEL_MARKET) then
		minBidPercent = self.db.profile.marketMinBidMul;
		buyoutPercent = self.db.profile.marketBuyoutMul;
	elseif (priceModel == self.PRIZE_MODEL_UNDERCUT) then
		minBidPercent = self.db.profile.undercutMinBidMul;	
		buyoutPercent = self.db.profile.undercutBuyoutMul;	
	elseif (priceModel == self.PRIZE_MODEL_MULTIPLIER) then
		minBidPercent = self.db.profile.multiplierMinBidMul;	
		buyoutPercent = self.db.profile.multiplierBuyoutMul;	
	elseif (priceModel == self.PRIZE_MODEL_LOWER) then
		minBidPercent = self.db.profile.lowerMinBidMul;	
		buyoutPercent = self.db.profile.lowerBuyoutMul;	
	end
	return minBidPercent, buyoutPercent
end

--[[
	Returns item's current price calculation for current auctions.
--]] 
local function _GetCurrentPrice(self, itemLink)
	local minBidPercent, buyoutPercent = _GetPriceModelModifier(self, self.PRIZE_MODEL_CURRENT)
	local minBid, buyout = vendor.Statistic:GetCurrentAuctionInfo(itemLink, vendor.AuctionHouse:IsNeutral())
	return _ModifyPrices(minBid, buyout, minBidPercent, buyoutPercent)
end

--[[
	Returns item's undercut calculation for current auctions.
--]] 
local function _GetUndercutPrice(self, itemLink)
	local minBidPercent, buyoutPercent = _GetPriceModelModifier(self, self.PRIZE_MODEL_UNDERCUT)
	local _, _, minBid, buyout = vendor.Statistic:GetCurrentAuctionInfo(itemLink, vendor.AuctionHouse:IsNeutral())
	return _ModifyPrices(minBid, buyout, minBidPercent, buyoutPercent)
end

--[[
	Returns item's mulltiplier calculation for current auctions.
--]] 
local function _GetMultiplierPrice(self, itemLink)
	local minBidPercent, buyoutPercent = _GetPriceModelModifier(self, self.PRIZE_MODEL_MULTIPLIER)
	local minBid = self.startPrize
	local buyout = self.startPrize
	return _ModifyPrices(minBid, buyout, minBidPercent, buyoutPercent)
end

--[[
	Returns item's market price calculation for current auctions.
--]] 
local function _GetMarketPrice(self, itemLink)
	local minBidPercent, buyoutPercent = _GetPriceModelModifier(self, self.PRIZE_MODEL_MARKET)
	local minBid, buyout = _GetItemAuctionAvgValues(itemLink)
	return _ModifyPrices(minBid, buyout, minBidPercent, buyoutPercent)
end

--[[
	Returns item's lower market price calculation for current auctions.
--]] 
local function _GetLowerPrice(self, itemLink)
	local minBidPercent, buyoutPercent = _GetPriceModelModifier(self, self.PRIZE_MODEL_LOWER)
	local result = vendor.Scanner:GetScanSet(vendor.AuctionHouse:IsNeutral(), itemLink)
	local minBid = nil
	local buyout = nil
	if (result and result:Size() > 0) then
		-- search for first auction above lower market threshold
		local avgMinBid, avgBuyout = _GetItemAuctionAvgValues(itemLink)
		if (avgBuyout and avgBuyout > 0) then
			local limit = avgBuyout * (self.db.profile.lowerMarketThreshold / 100.0)
			minBid = limit	
    		for i = 1, result:Size() do
    			local itemLinkKey, time, timeLeft, count, m, minIncrement, b = result:Get(i)
    			if (itemLinkKey) then
    				if (b and b > 0 and b > limit) then
    					if (not buyout or buyout > b) then
    						buyout = b
    					end 
    				end
    			end
    		end
		end
	end
	return _ModifyPrices(minBid, buyout, minBidPercent, buyoutPercent)
end

--[[
	Updates the deposit chown.
--]]
local function _UpdateDeposit(self)
	if (self.item) then
		local deposit = self.deposits[self.duration];
		local mult = 1;
		local stackCount = self.countDropDown:GetSelectedValue();
		if (type(stackCount) == "string") then
			-- sell all
			mult = self.item.count;
		else
			mult = self.stackDropDown:GetSelectedValue() * stackCount;
		end
		local copper = math.floor(mult * deposit + 0.5);
		MoneyFrame_Update("SalesFrameDeposit", copper);
	else
		MoneyFrame_Update("SalesFrameDeposit", 0);
	end
end

--[[
	Sets the given prizes for the item to be sold. 
--]]
local function _SetPrizes(self, minBid, buyout)
	MoneyInputFrame_SetCopper(self.startPriceBut, minBid);
	MoneyInputFrame_SetCopper(self.buyoutPriceBut, buyout);
	_UpdateDeposit(self);
end

--[[
	Sets the prizes for selling the item. 
--]]
local function _InitPrizes(self)
	vendor.Vendor:Debug("_InitPrizes");
	local minBid = 0;
	local buyout = 0;
	local mult = 1;
	if (self.item) then
		-- determine the multiplier for the displayed prize
		local bidType = self.bidTypeDropDown:GetSelectedValue();
		if (bidType == BID_TYPE_PER_STACK) then
			mult = self.stackDropDown:GetSelectedValue();
		elseif (bidType == BID_TYPE_ALL) then
			local count = self.countDropDown:GetSelectedValue();
			if (type(count) == "string") then
				-- sell all
				mult = self.item.count;
			else
				mult = self.stackDropDown:GetSelectedValue() * count;
			end
		end
		-- determine prizing model
		local prizeModel = self.prizingDropDown:GetSelectedValue();
		if (prizeModel == self.PRIZE_MODEL_FIX) then
			minBid, buyout = _GetItemAuctionSellValues(self.item.link);
		elseif (prizeModel == self.PRIZE_MODEL_MARKET) then
			minBid, buyout = _GetMarketPrice(self, self.item.link);
		elseif (prizeModel == self.PRIZE_MODEL_CURRENT) then
			minBid, buyout = _GetCurrentPrice(self, self.item.link);
		elseif (prizeModel == self.PRIZE_MODEL_UNDERCUT) then
			minBid, buyout = _GetUndercutPrice(self, self.item.link);
		elseif (prizeModel == self.PRIZE_MODEL_MULTIPLIER) then
			minBid, buyout = _GetMultiplierPrice(self, self.item.link);
		elseif (prizeModel == self.PRIZE_MODEL_LOWER) then
			minBid, buyout = _GetLowerPrice(self, self.item.link);
		end
		if (not minBid or minBid <= 0) then
			minBid = self.startPrize;
		end
		if (not buyout or buyout <= 0) then
			buyout = minBid * 2
		end
	end
	_SetPrizes(self, minBid * mult, (buyout or 0) * mult);
end

--[[
	Updates the count dropdown button
--]]
local function _UpdateCountDropDown(self)
	if (not self.item) then
		return;
	end
	local counts = {};
	local stackCount = self.stackDropDown:GetSelectedValue();
	local n = self.item.count / stackCount;
	if (n < 1) then
		n = 1;
	end
	n = math.floor(n);
	local hasAdd = self.item.count > n * stackCount;
	local select = n;
	if (hasAdd) then
		select = ""..n.."+";
	end
	local i = 1;
	local prev = nil;
	-- add all integral stack sizes
	while i <= n do
		table.insert(counts, i);
		prev = i;
		i = i * 2;
	end
	-- add the last if there are more items
	if (hasAdd or prev and prev < n) then
		if (hasAdd) then
			table.insert(counts, select);
		else
			table.insert(counts, n);
		end
	end
	self.countDropDown:SetItems(counts, select);
end

--[[
	Updates the dropdown buttons accoring to the selected item
--]]
local function _UpdateDropDowns(self)
	vendor.Vendor:Debug("_UpdateDropDowns enter")
	-- stacksize
	local stackSize = math.min(self.item.stackCount, self.item.count);
	local stackSizes = self.stackSizes[stackSize];
	if (not stackSizes) then
		stackSizes = {};
		self.stackSizes[stackSize] = stackSizes;
		local i = 1;
		local prev = nil;
		while i <= stackSize do
			table.insert(stackSizes, i);
			prev = i;
			if (i < 5) then
				i = i + 1;
			else
				i = i * 2;
			end
		end
		if (prev and prev < stackSize) then
			table.insert(stackSizes, stackSize);
		end
	end
	if (self.item.preferedStackSize > 0) then
		self.stackDropDown:SetItems(stackSizes, self.item.preferedStackSize);
	else
		self.stackDropDown:SetItems(stackSizes, stackSize);
	end
	-- stack count
	_UpdateCountDropDown(self);
	vendor.Vendor:Debug("_UpdateDropDowns exit")
end

--[[
	Shows information if mouse is over the selected item
--]]
local function _OnEnterItem(self, motion)
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
	if (self.ctrl.item) then
		GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
		GameTooltip.itemCount = self.ctrl.item.count
		GameTooltip:SetHyperlink(self.ctrl.item.link);
	else
		GameTooltip:SetText(L["Drag an item here to auction it"]);
	end
end

--[[
	Update the per item sell prizes and no. of stacks currently selected.
--]]
local function _UpdateSellInfo(self)
	local stackSize = self.stackDropDown:GetSelectedValue();
	self.minBidPerItem = MoneyInputFrame_GetCopper(self.startPriceBut);
	self.buyoutPerItem = MoneyInputFrame_GetCopper(self.buyoutPriceBut);
	local numStacks, items, _;
	local stackCount = self.countDropDown:GetSelectedValue();
	self.backlog = 0;
	if (type(stackCount) == "string") then
		-- we have a backdrop
		_, _, numStacks = string.find(stackCount, "^(%d+).*");
		self.backlog = self.item.count - (numStacks * stackSize);
		items = self.item.count;
	else
		numStacks = stackCount;
		items = numStacks * stackSize;
	end
	self.numStacks = numStacks;
	self.numItems = items;	
	local bidType = self.bidTypeDropDown:GetSelectedValue();
	if (bidType == BID_TYPE_PER_ITEM) then
		-- nothing to be done
	elseif bidType == BID_TYPE_PER_STACK then
		self.minBidPerItem = self.minBidPerItem / stackSize;
		self.buyoutPerItem = self.buyoutPerItem / stackSize;
	elseif bidType == BID_TYPE_ALL then
		self.minBidPerItem = self.minBidPerItem / items;
		self.buyoutPerItem = self.buyoutPerItem / items;			
	end
end

--[[
	Checks whether the current auction is valid and shows an error messages
	or enables the sell button accordingly.
--]]
local function _ValidateAuction(self)
	self.createBut:Show();
	self.createBut:Disable();
	self.buyoutErrorText:Hide();
	if (not self.item) then
		-- not item selected
		return;
	end
	local startPrize = MoneyInputFrame_GetCopper(self.startPriceBut);
	local buyoutPrize = MoneyInputFrame_GetCopper(self.buyoutPriceBut);
	-- Buyout price is less than the start price
	if (buyoutPrize > 0 and startPrize > buyoutPrize) then
		self.createBut:Hide();
		self.buyoutErrorText:Show();
		return;
	end
	-- Start price is 0 or greater than the max allowed
	if (startPrize < 1 or startPrize > MAXIMUM_BID_PRICE) then
		return;
	end
	_UpdateSellInfo(self);
	self.createBut:Enable();
end

--[[
	Auction duration radio button has been clicked
--]]
local function _DurationSelected(self, id)
	vendor.Vendor:Debug("duration selected");
	PlaySound("igMainMenuOptionCheckBoxOn");
	if (id == 1) then
		self.duration = DURATIONS[1];
		vendor.Vendor:Debug("short");
	elseif (id == 2) then
		self.duration = DURATIONS[2];
		vendor.Vendor:Debug("medium duration");
	else
		self.duration = DURATIONS[3];
		vendor.Vendor:Debug("long duration");
	end
	_UpdateDeposit(self);
end

--[[
	Updates the price modifier fields with the given value.
--]]
local function _UpdatePriceModifier(val1, val2)
	self.minBidMulEdit:SetText(val1)
	self.buyoutMulEdit:SetText(val2)
end

--[[
	Prizing model was selected.
--]]
local function _PrizingSelected(self, value)
	if (self.PRIZE_MODEL_CURRENT == value) then
		self.mulFrame:Show()
		_UpdatePriceModifier(self.db.profile.currentMinBidMul, self.db.profile.currentBuyoutMul)
	elseif (self.PRIZE_MODEL_MARKET == value) then
		self.mulFrame:Show()
		_UpdatePriceModifier(self.db.profile.marketMinBidMul, self.db.profile.marketBuyoutMul)
	elseif (self.PRIZE_MODEL_UNDERCUT == value) then
		self.mulFrame:Show()
		_UpdatePriceModifier(self.db.profile.undercutMinBidMul, self.db.profile.undercutBuyoutMul)	
	elseif (self.PRIZE_MODEL_MULTIPLIER == value) then
		self.mulFrame:Show()
		_UpdatePriceModifier(self.db.profile.multiplierMinBidMul, self.db.profile.multiplierBuyoutMul)	
	elseif (self.PRIZE_MODEL_LOWER == value) then
		self.mulFrame:Show()
		vendor.Vendor:Debug("update with: "..(self.db.profile.lowerMinBidMul or "NA"))
		_UpdatePriceModifier(self.db.profile.lowerMinBidMul, self.db.profile.lowerBuyoutMul)	
	else
		self.mulFrame:Hide()
	end
	_InitPrizes(self)
end

--[[
	Selects the given itemLink for selling
--]]
local function _SelectItem(self, itemLink)
	vendor.Vendor:Debug("_SelectItem enter")
	if (not itemLink) then
		SalesFrameItemName:SetText("");
		SalesFrameItemCount:Hide();
		SalesFrameItem:SetNormalTexture(nil);
		self.lastItem = self.item;
		self.item = nil;
	else
		self.auto:SetItem(itemLink)
		local name, link, stackCount, texture, count = self.inventory:GetItemInfo(itemLink);
		if (name) then
			SalesFrameItem:SetNormalTexture(texture);
			SalesFrameItemName:SetText(name);
			if (count > 1) then
				SalesFrameItemCount:SetText(count);
				SalesFrameItemCount:Show();
			else
				SalesFrameItemCount:Hide();
			end
			local itemLinkKey = vendor.Items:GetItemLinkKey(itemLink);
			local preferedStackSize = 0;
			local itemInfo = vendor.Items:GetItemInfo(itemLinkKey, vendor.AuctionHouse:IsNeutral());
			if (itemInfo) then
				vendor.Vendor:Debug("found itemInfo")
				if (self.db.profile.rememberStackSize and itemInfo.stackSize <= count) then
					preferedStackSize = itemInfo.stackSize;			
				end
				if (self.db.profile.rememberDuration and itemInfo.duration > 0) then
					vendor.Vendor:Debug("select duration: "..itemInfo.duration)
					self.durationDropDown:SelectValue(itemInfo.duration)
					_DurationSelected(self, itemInfo.duration)
				else
					vendor.Vendor:Debug("select default duration: %s", self.db.profile.defaultDuration)
					local defaultDuration = DURATION_CONSTS[self.db.profile.defaultDuration]
					vendor.Vendor:Debug("id: %d", defaultDuration)
					self.durationDropDown:SelectValue(defaultDuration)
					_DurationSelected(self, defaultDuration)
				end
				if (self.db.profile.rememberPriceModel and itemInfo.priceModel > 0) then
					vendor.Vendor:Debug("select priceModel: "..itemInfo.priceModel)
					self.prizingDropDown:SelectValue(itemInfo.priceModel)
					_PrizingSelected(self, itemInfo.priceModel)
				end
			end
			self.item = {name = name, link = link, stackCount = stackCount, texture = texture, count = count, preferedStackSize = preferedStackSize, itemLinkKey = itemLinkKey};
			_UpdateDropDowns(self);
		else
			-- item disappaered after it had been selected
			_SelectItem(self, nil);
		end
	end
	_InitPrizes(self);
	_ValidateAuction(self);
	vendor.Vendor:Debug("_SelectItem exit")
end

--[[
	Picks the given item for selling.
--]]
local function _PickItem(self, itemLink)
	-- use blizzard's auction slot to calculate deposit and start prize 
	ClickAuctionSellItemButton();
	local _, _, count, _, _, price = GetAuctionSellItemInfo();
	self.deposits = {};
	for _, duration in ipairs(DURATIONS) do
		self.deposits[duration] = CalculateAuctionDeposit(duration) / count;
	end;
	self.startPrize = price / count;
	_SelectItem(self, itemLink);
	-- now free blizzard's auction slot
	ClickAuctionSellItemButton();
	ClearCursor();
	-- display all known auctions about the item
	local scanSet = vendor.Scanner:GetScanSet(vendor.AuctionHouse:IsNeutral(), itemLink);
	self.scanResultFrame:Display(scanSet);
	vendor.Vendor:Debug("Check ScanSet freshness and size");
	if (scanSet:Size() == 0 or (scanSet:GetUpToDateness() >= self.db.profile.autoRefreshTime)) then
		vendor.Vendor:Debug("Refresh ScanSet");
		_RefreshClicked(self);
	end
end

--[[
	Picks the currently dragged item for selling.
--]]
local function _PickItemFromCursor(self)
	local type, _, itemLink = GetCursorInfo();
	if (type and type == "item" and itemLink) then
		_PickItem(self, itemLink)
	end
end

--[[
	Updates the list of inventory items.
--]]
local function _UpdateInventory(self)
	if (self.frame and self.frame:IsVisible()) then
		self.inventory:Update();
		if (self.item) then
			_SelectItem(self, self.item.link);
		end
	 end
end

--[[
	Rescan the extra information for prizing models.
--]]
local function _OnPriceModifierUpdate(self)
	local priceModel = self.prizingDropDown:GetSelectedValue()
	if (priceModel == self.PRIZE_MODEL_CURRENT) then
		self.db.profile.currentMinBidMul = self.minBidMulEdit:GetNumber()
		self.db.profile.currentBuyoutMul = self.buyoutMulEdit:GetNumber()
	elseif (priceModel == self.PRIZE_MODEL_MARKET) then
		self.db.profile.marketMinBidMul = self.minBidMulEdit:GetNumber()
		self.db.profile.marketBuyoutMul = self.buyoutMulEdit:GetNumber()
	elseif (priceModel == self.PRIZE_MODEL_UNDERCUT) then
		self.db.profile.undercutMinBidMul = self.minBidMulEdit:GetNumber()
		self.db.profile.undercutBuyoutMul = self.buyoutMulEdit:GetNumber()
	elseif (priceModel == self.PRIZE_MODEL_MULTIPLIER) then
		self.db.profile.multiplierMinBidMul = self.minBidMulEdit:GetNumber()
		self.db.profile.multiplierBuyoutMul = self.buyoutMulEdit:GetNumber()
	elseif (priceModel == self.PRIZE_MODEL_LOWER) then
		self.db.profile.lowerMinBidMul = self.minBidMulEdit:GetNumber()
		self.db.profile.lowerBuyoutMul = self.buyoutMulEdit:GetNumber()
	end
	_InitPrizes(self);
	
end

--[[
	Creates an auction with the currently selected item.
--]]
local function _CreateAuction(self)
	if (self.item) then
		local itemLink = self.item.link;
		local runTime = self.duration;
		local stackSize = self.stackDropDown:GetSelectedValue();
		
		-- remember several dropdown button values for next time
  		local itemInfo = vendor.Items:GetItemInfo(self.item.itemLinkKey, vendor.AuctionHouse:IsNeutral());
		if (not itemInfo) then
			itemInfo = vendor.Items:CreateItemInfo();
		end
		if (self.selectedStackSize) then
			-- a stackSize was selected manually, remember it
			itemInfo.stackSize = self.selectedStackSize;
		end
		itemInfo.priceModel = self.prizingDropDown:GetSelectedValue()
		itemInfo.duration = self.durationDropDown:GetSelectedValue()
 		vendor.Items:PutItemInfo(self.item.itemLinkKey, vendor.AuctionHouse:IsNeutral(), itemInfo);
		self.selectedStackSize = nil;
		
		-- create sell task
		_UpdateSellInfo(self);
		local task = vendor.SellTask:new(self.inventory, itemLink, self.minBidPerItem, self.buyoutPerItem, runTime, stackSize, self.numStacks, self.backlog);
		vendor.TaskQueue:AddTask(task);
	end
end

--[[
	Creates the frame for the minBid/buyout modifier calculation.
--]]
local function _CreateExtraPricingFrame(self)
	self.mulFrame = CreateFrame("Frame", "SalesFrameMulFrame", self.frame)
	self.minBidMulEdit = vendor.GuiTools.CreateWidget("EditBox", "SalesFrameMinBidMulEdit", self.mulFrame, "InputBoxTemplate", 60, 16, L["Percentage to be multiplied with the min bid price."])
	self.minBidMulEdit:SetPoint("LEFT", SalesFramePrizing, "RIGHT", 0, 18)
	self.minBidMulEdit.ctrl = self
	self.minBidMulEdit:SetNumeric(true)
	self.minBidMulEdit:SetAutoFocus(false)
	self.minBidMulEdit:SetScript("OnTextChanged", function(but) _OnPriceModifierUpdate(but.ctrl) end)
	f = self.mulFrame:CreateFontString("SalesFrameMinBidMulEditText", "ARTWORK", "GameFontHighlightSmall");
	f:SetText("%");
	f:SetPoint("LEFT", SalesFrameMinBidMulEdit, "RIGHT", 0, 0)
	
	self.buyoutMulEdit = vendor.GuiTools.CreateWidget("EditBox", "SalesFrameBuyoutMulEdit", self.mulFrame, "InputBoxTemplate", 60, 16, L["Percentage to be multiplied with the buyout price."])
	self.buyoutMulEdit:SetPoint("TOP", self.minBidMulEdit, "BOTTOM", 0, -5)
	self.buyoutMulEdit.ctrl = self
	self.buyoutMulEdit:SetNumeric(true)
	self.buyoutMulEdit:SetAutoFocus(false)
	self.buyoutMulEdit:SetScript("OnTextChanged", function(but) _OnPriceModifierUpdate(but.ctrl) end)
	f = self.mulFrame:CreateFontString("SalesFrameBuyoutMulEditText", "ARTWORK", "GameFontHighlightSmall");
	f:SetText("%");
	f:SetPoint("LEFT", SalesFrameBuyoutMulEdit, "RIGHT", 0, 0)

	self.mulFrame:Hide()
end

--[[
	Creates the auction buttons.
--]]
local function _CreateAuctionButtons(self)
	-- item to be sold
	local f = self.frame:CreateFontString("SalesFrameItemText", "ARTWORK", "GameFontHighlightSmall");
	f:SetText(L["Auction Item"]);
	f:SetPoint("TOPLEFT", 28, -83);
	self.itemBut = CreateFrame("Button", "SalesFrameItem", self.frame);
	self.itemBut.ctrl = self; 
	self.itemBut:SetWidth(37);
	self.itemBut:SetHeight(37);
	self.itemBut:SetPoint("TOPLEFT", 28, -98);
	local f = self.itemBut:CreateFontString("SalesFrameItemName", "BACKGROUND", "GameFontNormal");
	f:SetWidth(124);
	f:SetHeight(30);
	f:SetPoint("TOPLEFT", self.itemBut, "TOPRIGHT", 5, 0);
	local f = self.itemBut:CreateFontString("SalesFrameItemCount", "OVERLAY", "NumberFontNormal");
	f:SetJustifyH("RIGHT");
	f:SetPoint("BOTTOMRIGHT", -5, 2);
	self.itemBut:RegisterForDrag("LeftButton");
	self.itemBut:SetScript("OnClick", function(but) _PickItemFromCursor(but.ctrl); _ValidateAuction(but.ctrl); end);
	self.itemBut:SetScript("OnDragStart", function(but) _PickItemFromCursor(but.ctrl); _ValidateAuction(but.ctrl); end);
	self.itemBut:SetScript("OnReceiveDrag", function(but) _PickItemFromCursor(but.ctrl); end);
	self.itemBut:SetScript("OnEnter", _OnEnterItem);
	self.itemBut:SetScript("OnLeave", function() GameTooltip:Hide(); end);
	self.itemBut:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square");
	
	-- duration of auction
	vendor.Vendor:Debug("create durations")
	self.durationDropDown = vendor.DropDownButton:new("SalesFrameDuration", self.frame, 70, L["Auction Duration"], "");
	self.durationDropDown:SetPoint("TOPLEFT", 10, -150);
 	self.durationDropDown:SetId(DURATION_DROPDOWN_ID);
	self.durationDropDown:SetListener(self);
    local durations = {};
	table.insert(durations, {value = 1, text = "12 "..GetText("HOURS", nil, 12)});
	table.insert(durations, {value = 2, text = "24 "..GetText("HOURS", nil, 24)});
	table.insert(durations, {value = 3, text = "48 "..GetText("HOURS", nil, 48)});
	local defaultDuration = DURATION_CONSTS[self.db.profile.defaultDuration]
	self.durationDropDown:SetItems(durations, defaultDuration)
	self.duration = DURATIONS[defaultDuration]
	
	-- stacksize
	self.stackDropDown = vendor.DropDownButton:new("SalesFrameStackSize", self.frame, 70, L["Stack size"], L["Selects the size of the stacks to be sold."]);
	self.stackDropDown:SetPoint("TOPLEFT", 100, -150);
	self.stackDropDown:SetId(STACK_DROPDOWN_ID);
	self.stackDropDown:SetListener(self);
	
	-- count of "stacks"
	self.countDropDown = vendor.DropDownButton:new("SalesFrameCount", self.frame, 70, L["Amount"], L["Selects the number of stacks to be sold.\nThe number with the +-suffix sells\nalso any remaining items."]);
	self.countDropDown:SetPoint("TOPLEFT", SalesFrameStackSize, "BOTTOMLEFT", 0, -13);
 	self.countDropDown:SetId(COUNT_DROPDOWN_ID);
	self.countDropDown:SetListener(self);
	
	-- bid type (per item, per stack, ...)
	self.bidTypeDropDown = vendor.DropDownButton:new("SalesFrameBidType", self.frame, 70, L["Bid type"], L["Selects which prices should be shown in the bid and buyout input fields."]);
	self.bidTypeDropDown:SetPoint("TOPLEFT", SalesFrameDuration, "BOTTOMLEFT", 0, -13);
	self.bidTypeDropDown:SetId(BIDTYPE_DROPDOWN_ID);
	self.bidTypeDropDown:SetListener(self);
    local bidTypes = {};
	table.insert(bidTypes, {value = BID_TYPE_PER_ITEM, text = L["Per item"]});
	table.insert(bidTypes, {value = BID_TYPE_PER_STACK, text = L["Stack"]});
	table.insert(bidTypes, {value = BID_TYPE_ALL, text = L["Overall"]});
	self.bidTypeDropDown:SetItems(bidTypes, BID_TYPE_PER_ITEM);
	
	-- prizing model
	self.prizingDropDown = vendor.DropDownButton:new("SalesFramePrizing", self.frame, 70, L["Price calculation"], L["Selects the mode for calculating the sell prices.\nThe default mode Fixed price just select the last sell price."]);
	self.prizingDropDown:SetPoint("TOPLEFT", SalesFrameBidType, "BOTTOMLEFT", 0, -13);
 	self.prizingDropDown:SetId(PRIZING_DROPDOWN_ID);
	self.prizingDropDown:SetListener(self);
    local prizeModels = {};
	table.insert(prizeModels, {value = self.PRIZE_MODEL_FIX, text = L["Fixed price"]});
	table.insert(prizeModels, {value = self.PRIZE_MODEL_CURRENT, text = L["Current price"]});
	table.insert(prizeModels, {value = self.PRIZE_MODEL_UNDERCUT, text = L["Undercut"]})
	table.insert(prizeModels, {value = self.PRIZE_MODEL_LOWER, text = L["Lower market threshold"]})
	table.insert(prizeModels, {value = self.PRIZE_MODEL_MULTIPLIER, text = L["Selling price"]})
	table.insert(prizeModels, {value = self.PRIZE_MODEL_MARKET, text = L["Market price"]});
	self.prizingDropDown:SetItems(prizeModels, self.PRIZE_MODEL_FIX);
		
	-- extra config of prizing model
	_CreateExtraPricingFrame(self)
	
--	self.autoBut = CreateFrame("Button", "VendorAutomaticPriceModelFrameIcon", self.frame)
--	self.autoBut.ctrl = self
--	self.autoBut:SetWidth(37)
--	self.autoBut:SetHeight(37)
--	self.autoBut:SetPoint("TOPLEFT", 0, 0)
--	self.autoBut:SetNormalTexture("Interface/Icons/Ability_GolemThunderClap")
--	f = self.autoBut:CreateFontString("VendorAutomaticPriceModelFrameName", "BACKGROUND", "GameFontNormal")
--	f:SetWidth(124)
--	f:SetHeight(30)
--	f:SetPoint("TOPLEFT", self.autoBut, "TOPRIGHT", 5, 0)
--	f:SetText("BLUBBBBS")


	self.auto:CreateFrame(self.frame)
	
	-- starting prize
	f = self.frame:CreateFontString("SalesFramePriceText", "ARTWORK", "GameFontHighlightSmall");
	f:SetText(L["Starting Price"]);
	if ( GetLocale() == "zhTW" or GetLocale() == "zhCN" ) then
		f:SetPoint("TOPLEFT", 28, -278);
	else
		f:SetPoint("TOPLEFT", 28, -283);
	end
	self.startPriceBut = CreateFrame("Frame", "SalesFrameStartPrice", self.frame, "MoneyInputFrameTemplate");
	self.startPriceBut:SetPoint("TOPLEFT", SalesFramePriceText, "BOTTOMLEFT", 3, -2);
	self.startPriceBut.controller = self;

	local frameName = self.startPriceBut:GetName();
	local goldBut = getglobal(frameName.."GoldButton");
	MoneyInputFrame_SetOnValueChangedFunc(self.startPriceBut, function() _ValidateAuction(self) end);
	SalesFrameStartPriceGold:SetMaxLetters(6);
	
	-- buyout prize
	f = self.frame:CreateFontString("SalesFrameBuyoutText", "ARTWORK", "GameFontHighlightSmall");
	f:SetText(L["Buyout Price"].." |cff808080("..L["optional"]..")|r");
	if ( GetLocale() == "zhTW" or GetLocale() == "zhCN" ) then
		f:SetPoint("TOPLEFT", SalesFramePriceText, 0, -35);
	else
		f:SetPoint("TOPLEFT", SalesFramePriceText, 0, -30);
	end
	self.buyoutPriceBut = CreateFrame("Frame", "SalesFrameBuyoutPrice", self.frame, "MoneyInputFrameTemplate");
	--- ?????? why not ????? self.buyoutPriceBut:SetPoint("TOPLEFT", SalesFrameBuyoutText, "BOTTOMLEFT", 3, -2);
	if ( GetLocale() == "zhTW" or GetLocale() == "zhCN" ) then
		self.buyoutPriceBut:SetPoint("TOPLEFT", 31, -329);
	else
		self.buyoutPriceBut:SetPoint("TOPLEFT", 31, -324);
	end
	self.buyoutPriceBut.controller = self;
	MoneyInputFrame_SetOnValueChangedFunc(self.buyoutPriceBut, function() _ValidateAuction(self) end);
	SalesFrameBuyoutPriceGold:SetMaxLetters(6);

	-- Set focus rules
	MoneyInputFrame_SetPreviousFocus(SalesFrameStartPrice, SalesFrameBuyoutPriceCopper);
	MoneyInputFrame_SetNextFocus(SalesFrameStartPrice, SalesFrameBuyoutPriceGold);
	MoneyInputFrame_SetPreviousFocus(SalesFrameBuyoutPrice, SalesFrameStartPriceCopper);
	MoneyInputFrame_SetNextFocus(SalesFrameBuyoutPrice, SalesFrameStartPriceGold);
	
	-- total deposit
	f = self.frame:CreateFontString("SalesFrameDepositText", "ARTWORK", "GameFontNormal");
	f:SetText(L["Deposit:"]);
	f:SetPoint("TOPLEFT", 28, -364);
	
	self.depositBut = CreateFrame("Frame", "SalesFrameDeposit", self.frame, "DepositFrameTemplate");
	self.depositBut:SetPoint("LEFT", SalesFrameDepositText, "RIGHT", 5, 0);
	self.depositBut.controller = self;
	
	-- up-to-dateness of the item list
	f = self.frame:CreateFontString("SalesFrameUpToDatenessText", "ARTWORK", "GameFontNormal");
	f:SetText("");
	f:SetPoint("BOTTOMLEFT", 300, 20);
	self.statusText = f;
	
	-- auction creation
	self.createBut = CreateFrame("Button", "SalesFrameCreate", self.frame, "UIPanelButtonTemplate");
	self.createBut:SetText(L["Create Auction"]);
	self.createBut:SetWidth(191);
	self.createBut:SetHeight(20);
	self.createBut:SetPoint("BOTTOMLEFT", 18, 39);
	self.createBut.ctrl = self;
	self.createBut:SetScript("OnClick", function(but) _CreateAuction(but.ctrl) end);
	
	-- auction creation not possible error text
	self.buyoutErrorText = self.frame:CreateFontString("SalesFrameBuyoutErrorText", "ARTWORK", "GameFontRedSmall");
	self.buyoutErrorText:SetText(L["Buyout < bid"]);
	self.buyoutErrorText:SetPoint("TOPLEFT", self.createBut, "TOPLEFT", 15, -5);
end

--[[
	Checks whether the refresh button should be enabled.
--]] 
local function _OnUpdateRefreshButton(self)
	if (self.item) then
		self.refreshBut:Enable()
	else
		self.refreshBut:Disable()
	end
end

--[[
	Creates the central close etc. buttons.
--]]
local function _CreateFrameButtons(self)
	self.closeBut = vendor.AuctionHouse:CreateCloseButton(self.frame, "SalesFrameClose");
	self.refreshBut = vendor.GuiTools.CreateButton(self.frame, "UIPanelButtonTemplate", 86, 22, L["Refresh"], L["Refreshes the list of current auctions for the selected item to be sold."]);
	self.refreshBut.ctrl = self;
	self.refreshBut:SetScript("OnUpdate", function(but) _OnUpdateRefreshButton(but.ctrl) end) 
	CreateFrame("Button", "SalesFrameRefresh", self.frame, "UIPanelButtonTemplate");
	self.refreshBut:SetText(L["Refresh"]);
	self.refreshBut:SetWidth(86);
	self.refreshBut:SetHeight(22);
	self.refreshBut:SetPoint("BOTTOMLEFT", 212, 14);
	self.refreshBut:SetScript("OnClick", function(but) _RefreshClicked(but.ctrl) end);
end

--[[
	Sets the given value in the profile.
--]]
local function _SetValue(field, value)
	self.db.profile[field] = value;
	if (field == "pickupByClick") then
		if (value and not self:IsHooked("ContainerFrameItemButton_OnModifiedClick")) then
			self:Hook("ContainerFrameItemButton_OnModifiedClick", true)
		elseif (not value and self:IsHooked("ContainerFrameItemButton_OnModifiedClick")) then
			self:Unhook("ContainerFrameItemButton_OnModifiedClick")
		end	
	end
end

--[[
	Returns the given value from the profile.
--]]
local function _GetValue(field)
	return self.db.profile[field]
end

--[[
	Options for this module.
--]]
local function _CreateOptions()
	vendor.Vendor.options.args.Seller = {
		type = "group",
		name = L["Seller"],
		desc = L["Seller"],
		args = {
			autoMode ={
				type = "group",
				name = L["Auto selling"],
				desc = L["Settings for automatically selecting the best fitting price model."],
				order = 1,
				args = {
					aboveMarketThreshold = {
						type = "range",
        				name = L["Upper market threshold"],
        				desc = L["Minimal needed percentage of buyouts compared to market price, until they are assumed to be considerably above the market price."],
        				min = 101,
        				max = 500,
        				step = 1,
        				get = _GetValue,
        				set = _SetValue,
        				passValue = "upperMarketThreshold",
        				order = 1,
        			},
					underMarketThreshold = {
						type = "range",
        				name = L["Lower market threshold"],
        				desc = L["Maximal allowed percentage of buyouts compared to market price, until they are assumed to be considerably under the market price."],
        				min = 1,
        				max = 99,
        				step = 1,
        				get = _GetValue,
        				set = _SetValue,
        				passValue = "lowerMarketThreshold",
        				order = 2,
        			},
        		},
        	},
			autoRefreshTime = {
				type = "range",
				name = L["Autorefresh time"],
				desc = L["Selects the minimum time in seconds that has to be passed, before the auction house is automatically scanned for the item to be sold."],
				min = 10,
				max = 360,
				step = 1,
				get = _GetValue,
				set = _SetValue,
				passValue = "autoRefreshTime",
				order = 2,
			},
			pickupByClick = {
				type = "toggle",
				name = L["Pickup by click"],
				desc = L["Pickup items to be soled, when they are shift left clicked."],
				get = _GetValue,
				set = _SetValue,
				passValue = "pickupByClick",
				order = 3,
			},
			rememberStackSize = {
				type = "toggle",
				name = L["Remember stacksize"],
				desc = L["Automatically selects the stacksize used the last time for a given item."],
				get = _GetValue,
				set = _SetValue,
				passValue = "rememberStackSize",
				order = 4,
			},
			rememberDuration = {
				type = "toggle",
				name = L["Remember duration"],
				desc = L["Automatically selects the duration used the last time for a given item."],
				get = _GetValue,
				set = _SetValue,
				passValue = "rememberDuration",
				order = 5,
			},
			defaultDuration = {
				type = "text",
				name = L["Default duration"],
				desc = L["Default duration for creating auctions."],
				get = _GetValue,
				set = _SetValue,
				validate = {"12h", "24h", "48h"}, 
				passValue = "defaultDuration",
				order = 6,
			},																		
			rememberPriceModel = {
				type = "toggle",
				name = L["Remember price model"],
				desc = L["Automatically selects the price model used the last time for a given item."],
				get = _GetValue,
				set = _SetValue,
				passValue = "rememberPriceModel",
				order = 7,
			},									
		}
	}
end

--[[
	Initializes the module.
--]]
function vendor.Seller:OnInitialize()
	self.db = vendor.Vendor:AcquireDBNamespace("Seller");
	vendor.Vendor:RegisterDefaults("Seller", "profile", {
		autoRefreshTime = 60,
		marketMinBidMul = 150,
		marketBuyoutMul = 150,
		currentMinBidMul = 100,
		currentBuyoutMul = 100,
		undercutMinBidMul = 98,
		undercutBuyoutMul = 98,
		lowerMinBidMul = 98,
		lowerBuyoutMul = 98,
		multiplierMinBidMul = 200,
		multiplierBuyoutMul = 250,
		pickupByClick = true,
		rememberStackSize = true,
		rememberDuration = true,
		rememberPriceModel = true,
		upperMarketThreshold = 200,
		lowerMarketThreshold = 30,
		autoPriceModel = true,		
		defaultDuration = "48h",
	});
	_CreateOptions();
	self.auto = vendor.AutomaticPriceModel:new()	
end

--[[
	Initializes the module.
--]]
function vendor.Seller:OnEnable()
	self.isScanning = false;
	self.stackSizes = {};
	self.inventory = vendor.InventoryHandle:new();
	self.lastItem = nil;
	self:RegisterEvent("BAG_UPDATE");
	self:RegisterEvent("AUCTION_HOUSE_CLOSED");
	self:RegisterEvent("NEW_AUCTION_UPDATE");
   	self:Hook("StartAuction", true);
	if (self.db.profile.pickupByClick) then
		self:Hook("ContainerFrameItemButton_OnModifiedClick", true)
	end
	-- register for new ScanResults
	vendor.Scanner:AddScanSnapshotListener(self);
	self.frame = vendor.AuctionHouse:CreateTabFrame("AuctionFrameVendor", "AuctionMaster", L["Sell"], self);
	--self.vendorTabButton = vendor.AuctionHouse:CreateTabButton(L["Seller"], 4);
	_CreateFrameButtons(self);
	_CreateAuctionButtons(self);
	self.scanResultFrame = vendor.CompactScanResultFrame:new("SalesScanResultFrame", self.frame, 15, self.statusText);
	self.scanResultFrame:SetPoint("TOPLEFT", 25, 33);
end

--[[
	Updates the gui for displaying the frame (Interface method).
--]]
function vendor.Seller:UpdateTabFrame()
	vendor.Vendor:Debug("Seller:UpdateTabFrame enter")
	AuctionFrameTopLeft:SetTexture("Interface\\Addons\\AuctionMaster\\Media\\UI-AuctionFrame-Auction-TopLeft");
	AuctionFrameTop:SetTexture("Interface\\AuctionFrame\\UI-AuctionFrame-Auction-Top");
	AuctionFrameTopRight:SetTexture("Interface\\AuctionFrame\\UI-AuctionFrame-Auction-TopRight");
	AuctionFrameBotLeft:SetTexture("Interface\\AuctionFrame\\UI-AuctionFrame-Auction-BotLeft");
	AuctionFrameBot:SetTexture("Interface\\AuctionFrame\\UI-AuctionFrame-Auction-Bot");
	AuctionFrameBotRight:SetTexture("Interface\\Addons\\AuctionMaster\\Media\\UI-AuctionFrame-Auction-BotRight");
	self.auto:SetItem(nil)
	vendor.Vendor:Debug("Seller:UpdateTabFrame exit")
end

--[[
	Returns the type of this auction house tab.
--]]
function vendor.Seller:GetTabType()
	return SELLER_TAB;
end
	
--[[
	Shows the tabbed frame (Interface method).
--]]
function vendor.Seller:ShowTabFrame()
	vendor.Vendor:Debug("Seller:ShowTabFrame enter")
	self.frame:Show();
	_UpdateInventory(self);
	self.scanResultFrame:Update();
	vendor.Vendor:Debug("Seller:ShowTabFrame exit")
end

--[[
	Hides the tabbed frame (Interface method).
--]]
function vendor.Seller:HideTabFrame()
	vendor.Vendor:Debug("Seller:HideTabFrame enter")
	self.frame:Hide();
	vendor.Vendor:Debug("Seller:HideTabFrame exit")
end

--[[
	Interface method for DropDownButton.
--]]
function vendor.Seller:DropDownButtonSelected(button, value)
	vendor.Vendor:Debug("dropdown selected");
	local id = button:GetId();
	vendor.Vendor:Debug("id: "..id);	
	if (id == DURATION_DROPDOWN_ID) then
		vendor.Vendor:Debug("duration selected");
		-- duration has been selected
		_DurationSelected(self, value)
	elseif (id == PRIZING_DROPDOWN_ID) then
		vendor.Vendor:Debug("prizing selected");
		_PrizingSelected(self, value)
	elseif (self.item) then
		local bidType = self.bidTypeDropDown:GetSelectedValue();
		local stackSize = self.stackDropDown:GetSelectedValue();
		if (id == STACK_DROPDOWN_ID) then
			vendor.Vendor:Debug("stack selected");
			-- new stacksize has been selected
			_UpdateCountDropDown(self);
			self.selectedStackSize = stackSize;
			-- only change the prizes, if they are shown per stack
			if (bidType == BID_TYPE_PER_STACK) then
				_SetPrizes(self, self.minBidPerItem * stackSize, self.buyoutPerItem * stackSize);
			end
		elseif (id == BIDTYPE_DROPDOWN_ID) then
			vendor.Vendor:Debug("bidtype selected");
			-- change prizes according to stacksize/numItems
			if (bidType == BID_TYPE_PER_ITEM) then
				_SetPrizes(self, self.minBidPerItem, self.buyoutPerItem);
			elseif (bidType == BID_TYPE_PER_STACK) then
				_SetPrizes(self, self.minBidPerItem * stackSize, self.buyoutPerItem * stackSize);
			elseif (bidType == BID_TYPE_ALL) then
				_SetPrizes(self, self.minBidPerItem * self.numItems, self.buyoutPerItem * self.numItems);
			end
		elseif (id == COUNT_DROPDOWN_ID) then
			vendor.Vendor:Debug("count selected");
			-- change only needed if bid type is set to all
			if (bidType == BID_TYPE_ALL) then
				_SetPrizes(self, self.minBidPerItem * self.numItems, self.buyoutPerItem * self.numItems);
			end
		end
		_UpdateDeposit(self);
	end
end

--[[
	BAG_UPDATe event has been triggered.
--]]
function vendor.Seller:BAG_UPDATE()
	_UpdateInventory(self);
end

--[[
	Auction house has been closed
--]]
function vendor.Seller:AUCTION_HOUSE_CLOSED()
	_SelectItem(self, nil);
end

--[[
	Hooks the StartAuction function to remember the prizes.
--]]
function vendor.Seller:StartAuction(minBid, buyoutPrize, runTime)
   	-- we want to remember the "minBid" and "buyoutPrice" for the next time
   	local name, texture, count, quality, canUse, price = GetAuctionSellItemInfo();
   	if (name) then
      	-- unfortunately we need the itemLink, we have to scan the inventory
      	local itemLink = _FindInventoryItemLink(name);
      	if (itemLink) then
      		local itemLinkKey = vendor.Items:GetItemLinkKey(itemLink);
      		local itemInfo = vendor.Items:GetItemInfo(itemLinkKey, vendor.AuctionHouse:IsNeutral());
			if (not itemInfo) then
				itemInfo = vendor.Items:CreateItemInfo();
			end
	 		itemInfo.minBid = math.floor(minBid / count);
	 		itemInfo.buyout = math.floor(buyoutPrize / count);
	 		vendor.Items:PutItemInfo(itemLinkKey, vendor.AuctionHouse:IsNeutral(), itemInfo);
      	end
   	end
   	self.hooks.StartAuction(minBid, buyoutPrize, runTime);
end

--[[
	Event callback for filling in last auction prizes for the given item.
--]]
function vendor.Seller:NEW_AUCTION_UPDATE()
	MoneyInputFrame_SetCopper(StartPrice, 0);
	MoneyInputFrame_SetCopper(BuyoutPrice, 0);
	-- we want to fill in old "minBid" and "buyoutPrice"
	local name, texture, count, quality, canUse, price = GetAuctionSellItemInfo();
	if (name) then
		-- unfortunately we need the itemLink, we have to scan the inventory
		local itemLink = _FindInventoryItemLink(name);
		if (itemLink) then
			local itemInfo = vendor.Items:GetItemInfo(itemLinkKey, vendor.AuctionHouse:IsNeutral());
			if (itemInfo) then
				if (itemInfo.minBid > 0) then
					MoneyInputFrame_SetCopper(StartPrice, itemInfo.minBid * count);
	 			end
	 			if (itemInfo.buyout > 0) then
	    			MoneyInputFrame_SetCopper(BuyoutPrice, itemInfo.buyout * count);
	 			end
      		end
      	end
   	end
end

--[[
	Returns information about the currently selected item, if any.
	@return itemName, itemLink
--]]
function vendor.Seller:GetSelectedItemInfo()
	if (self.item) then
		return self.item.name, self.item.link;
	end
	return nil;
end	

--[[
	ScanResultListener interface method for new or updated
	Scanresults.
--]]
function vendor.Seller:ScanSnapshotUpdated(scanSnapshot, isNeutral)
	vendor.Vendor:Debug("vendor.Seller:ScanSnapshotUpdated enter");
	local prizeModel = self.prizingDropDown:GetSelectedValue();
	if (prizeModel ~= self.PRIZE_MODEL_FIX) then
		self.auto:Update()
		_InitPrizes(self);
	end
	vendor.Vendor:Debug("vendor.Seller:ScanSnapshotUpdated exit");
end

--[[
	Returns whether the seller frame is currently visible.
--]]
function vendor.Seller:IsSellerFrameVisible()
	return vendor.AuctionHouse:IsAuctionHouseTabShown(SELLER_TAB)
end

--[[
	Inventory item was clicked, we pick it up for the seller frame, if appropriate. 
--]]
function vendor.Seller:ContainerFrameItemButton_OnModifiedClick(frame, button, ...)
	vendor.Vendor:Debug("ContainerFrameItemButton_OnModifiedClick")
	-- some addons forget to pass the button parameter :-(
	if ((not button or button == "LeftButton") and IsShiftKeyDown() and self:IsSellerFrameVisible() and not CursorHasItem()) then
		vendor.Vendor:Debug("pickup item")
		local itemLink = GetContainerItemLink(frame:GetParent():GetID(), frame:GetID())
		if (itemLink) then
			-- we have to pickup for deposit calculation
			PickupContainerItem(frame:GetParent():GetID(), frame:GetID())
			_PickItem(self, itemLink)
		else
			vendor.Vendor:Debug("no itemlink")
		end
	else
		vendor.Vendor:Debug("don't pick item'")
		self.hooks["ContainerFrameItemButton_OnModifiedClick"](frame, button, ...)
	end
end

--[[
	Selects the given pricing model.
--]]
function vendor.Seller:SelectPricingModel(id)
	vendor.Vendor:Debug("Seller:SelectPricingModel enter id: "..(id or "NA"))
	self.prizingDropDown:SelectValue(id)
	_PrizingSelected(self, id)
	vendor.Vendor:Debug("Seller:SelectPricingModel exit")
end
